@@ -193,18 +193,18 @@ EQPrefs         theEQPrefs =
             0x00000000,                 // reserved     = 0 
             0x0000000C,                 // filterCount  = 12
             {                           // filter coefficients in sets of:  b0, b1, b2, a1, a2 and are in 4.20 format
-                {{{ 0x03, 0xD3, 0xD3 }, { 0x07, 0xA7, 0xA7 }, { 0x03, 0xD3, 0xD3 }, { 0xFC, 0x6C, 0x4D }, { 0x02, 0xE3, 0x01 }}},
-                {{{ 0x10, 0x11, 0xF1 }, { 0xE0, 0x49, 0x4B }, { 0x0F, 0xA5, 0xBD }, { 0xE0, 0x49, 0x4B }, { 0x0F, 0xB7, 0xAE }}},
-                {{{ 0x0F, 0xF3, 0xF4 }, { 0xE0, 0x63, 0x53 }, { 0x0F, 0xAE, 0xEA }, { 0xE0, 0x63, 0x53 }, { 0x0F, 0xA2, 0xDE }}},
-                {{{ 0x0F, 0x03, 0x89 }, { 0xE3, 0x69, 0xA5 }, { 0x0D, 0xD9, 0x08 }, { 0xE3, 0x69, 0xA5 }, { 0x0C, 0xDC, 0x91 }}},
-                {{{ 0x13, 0x53, 0x5B }, { 0xF2, 0x38, 0x8C }, { 0x04, 0xF5, 0x87 }, { 0xF2, 0x38, 0x8C }, { 0x08, 0x48, 0xE3 }}},
-                {{{ 0x0A, 0xCE, 0x4E }, { 0x04, 0xC8, 0x40 }, { 0x06, 0xBA, 0xA5 }, { 0x04, 0xC8, 0x40 }, { 0x01, 0x88, 0xF4 }}},
-                {{{ 0x03, 0xD3, 0xD3 }, { 0x07, 0xA7, 0xA7 }, { 0x03, 0xD3, 0xD3 }, { 0xFC, 0x6C, 0x4D }, { 0x02, 0xE3, 0x01 }}},
-                {{{ 0x10, 0x11, 0xF1 }, { 0xE0, 0x49, 0x4B }, { 0x0F, 0xA5, 0xBD }, { 0xE0, 0x49, 0x4B }, { 0x0F, 0xB7, 0xAE }}},
-                {{{ 0x0F, 0xF3, 0xF4 }, { 0xE0, 0x63, 0x53 }, { 0x0F, 0xAE, 0xEA }, { 0xE0, 0x63, 0x53 }, { 0x0F, 0xA2, 0xDE }}},
-                {{{ 0x0F, 0x03, 0x89 }, { 0xE3, 0x69, 0xA5 }, { 0x0D, 0xD9, 0x08 }, { 0xE3, 0x69, 0xA5 }, { 0x0C, 0xDC, 0x91 }}},
-                {{{ 0x13, 0x53, 0x5B }, { 0xF2, 0x38, 0x8C }, { 0x04, 0xF5, 0x87 }, { 0xF2, 0x38, 0x8C }, { 0x08, 0x48, 0xE3 }}},
-                {{{ 0x0A, 0xCE, 0x4E }, { 0x04, 0xC8, 0x40 }, { 0x06, 0xBA, 0xA5 }, { 0x04, 0xC8, 0x40 }, { 0x01, 0x88, 0xF4 }}}
+                {{{ 0x02, 0xEC, 0xAE }, { 0x05, 0xD9, 0x5C }, { 0x02, 0xEC, 0xA5 }, { 0xFB, 0x5C, 0xAE }, { 0x00, 0x56, 0x0A }}},
+                {{{ 0x0F, 0xF9, 0x5A }, { 0xE0, 0x82, 0x03 }, { 0x0F, 0x8C, 0x61 }, { 0xE0, 0x82, 0x03 }, { 0x0F, 0x85, 0xBB }}},
+                {{{ 0x10, 0x09, 0x23 }, { 0xE0, 0x50, 0x40 }, { 0x0F, 0xA7, 0x79 }, { 0xE0, 0x50, 0x40 }, { 0x0F, 0xB0, 0x9D }}},
+                {{{ 0x0F, 0x9F, 0x7C }, { 0xE2, 0x51, 0xDA }, { 0x0E, 0x69, 0x0E }, { 0xE2, 0x51, 0xDA }, { 0x0E, 0x08, 0x8A }}},
+                {{{ 0x14, 0x36, 0x79 }, { 0xF3, 0xAC, 0x86 }, { 0x03, 0xB2, 0xFF }, { 0xF3, 0xAC, 0x86 }, { 0x07, 0xE9, 0x78 }}},
+                {{{ 0x0D, 0x3A, 0x3A }, { 0x06, 0xFD, 0xB0 }, { 0x08, 0x5A, 0xCA }, { 0x06, 0xFD, 0xB0 }, { 0x05, 0x95, 0x05 }}},
+                {{{ 0x02, 0xEC, 0xAE }, { 0x05, 0xD9, 0x5C }, { 0x02, 0xEC, 0xA5 }, { 0xFB, 0x5C, 0xAE }, { 0x00, 0x56, 0x0A }}},
+                {{{ 0x0F, 0xF9, 0x5A }, { 0xE0, 0x82, 0x03 }, { 0x0F, 0x8C, 0x61 }, { 0xE0, 0x82, 0x03 }, { 0x0F, 0x85, 0xBB }}},
+                {{{ 0x10, 0x09, 0x23 }, { 0xE0, 0x50, 0x40 }, { 0x0F, 0xA7, 0x79 }, { 0xE0, 0x50, 0x40 }, { 0x0F, 0xB0, 0x9D }}},
+                {{{ 0x0F, 0x9F, 0x7C }, { 0xE2, 0x51, 0xDA }, { 0x0E, 0x69, 0x0E }, { 0xE2, 0x51, 0xDA }, { 0x0E, 0x08, 0x8A }}},
+                {{{ 0x14, 0x36, 0x79 }, { 0xF3, 0xAC, 0x86 }, { 0x03, 0xB2, 0xFF }, { 0xF3, 0xAC, 0x86 }, { 0x07, 0xE9, 0x78 }}},
+                {{{ 0x0D, 0x3A, 0x3A }, { 0x06, 0xFD, 0xB0 }, { 0x08, 0x5A, 0xCA }, { 0x06, 0xFD, 0xB0 }, { 0x05, 0x95, 0x05 }}}
             }                           // end of filter set
         },
         {                               // ----------     Equalization Set #8     ----------
@@ -275,17 +275,17 @@ EQPrefs         theEQPrefs =
             0x0000000C,                 // filterCount  = 12
             {                           // filter coefficients in sets of:  b0, b1, b2, a1, a2 and are in 4.20 format
                 {{{ 0x03, 0xD3, 0xD3 }, { 0x07, 0xA7, 0xA7 }, { 0x03, 0xD3, 0xD3 }, { 0xFC, 0x6C, 0x4D }, { 0x02, 0xE3, 0x01 }}},
-                {{{ 0x10, 0x1A, 0x9D }, { 0xE0, 0x49, 0x03 }, { 0x0F, 0x9D, 0x3D }, { 0xE0, 0x49, 0x03 }, { 0x0F, 0xB7, 0xDA }}},
-                {{{ 0x0F, 0xF2, 0xAB }, { 0xE0, 0x62, 0x85 }, { 0x0F, 0xB2, 0x0A }, { 0xE0, 0x62, 0x85 }, { 0x0F, 0xA4, 0xB5 }}},
-                {{{ 0x0F, 0x78, 0xB5 }, { 0xE2, 0x58, 0xAD }, { 0x0E, 0x74, 0xF8 }, { 0xE2, 0x58, 0xAD }, { 0x0D, 0xED, 0xAD }}},
-                {{{ 0x13, 0x39, 0x75 }, { 0xF2, 0x01, 0x72 }, { 0x05, 0x1A, 0xF8 }, { 0xF2, 0x01, 0x72 }, { 0x08, 0x54, 0x6E }}},
-                {{{ 0x0B, 0x3D, 0xDB }, { 0x05, 0xE1, 0x85 }, { 0x06, 0xE9, 0xE6 }, { 0x05, 0xE1, 0x85 }, { 0x02, 0x27, 0xC2 }}},
+                {{{ 0x10, 0x0C, 0xDD }, { 0xE0, 0x41, 0xC7 }, { 0x0F, 0xB2, 0x3A }, { 0xE0, 0x41, 0xC7 }, { 0x0F, 0xBF, 0x17 }}},
+                {{{ 0x0F, 0xB9, 0xFB }, { 0xE2, 0x0A, 0x1C }, { 0x0E, 0x95, 0xB4 }, { 0xE2, 0x0A, 0x1C }, { 0x0E, 0x4F, 0xB0 }}},
+                {{{ 0x0F, 0x46, 0xC0 }, { 0xE7, 0x71, 0x2C }, { 0x0D, 0x6A, 0xB6 }, { 0xE7, 0x71, 0x2C }, { 0x0C, 0xB1, 0x77 }}},
+                {{{ 0x13, 0x14, 0xD6 }, { 0xF4, 0x40, 0x0B }, { 0x04, 0xC9, 0x95 }, { 0xF4, 0x40, 0x0B }, { 0x07, 0xDE, 0x6C }}},
+                {{{ 0x0D, 0x4B, 0x9B }, { 0x06, 0x86, 0xF2 }, { 0x08, 0x37, 0xCD }, { 0x06, 0x86, 0xF2 }, { 0x05, 0x83, 0x68 }}},
                 {{{ 0x03, 0xD3, 0xD3 }, { 0x07, 0xA7, 0xA7 }, { 0x03, 0xD3, 0xD3 }, { 0xFC, 0x6C, 0x4D }, { 0x02, 0xE3, 0x01 }}},
-                {{{ 0x10, 0x1A, 0x9D }, { 0xE0, 0x49, 0x03 }, { 0x0F, 0x9D, 0x3D }, { 0xE0, 0x49, 0x03 }, { 0x0F, 0xB7, 0xDA }}},
-                {{{ 0x0F, 0xF2, 0xAB }, { 0xE0, 0x62, 0x85 }, { 0x0F, 0xB2, 0x0A }, { 0xE0, 0x62, 0x85 }, { 0x0F, 0xA4, 0xB5 }}},
-                {{{ 0x0F, 0x78, 0xB5 }, { 0xE2, 0x58, 0xAD }, { 0x0E, 0x74, 0xF8 }, { 0xE2, 0x58, 0xAD }, { 0x0D, 0xED, 0xAD }}},
-                {{{ 0x13, 0x39, 0x75 }, { 0xF2, 0x01, 0x72 }, { 0x05, 0x1A, 0xF8 }, { 0xF2, 0x01, 0x72 }, { 0x08, 0x54, 0x6E }}},
-                {{{ 0x0B, 0x3D, 0xDB }, { 0x05, 0xE1, 0x85 }, { 0x06, 0xE9, 0xE6 }, { 0x05, 0xE1, 0x85 }, { 0x02, 0x27, 0xC2 }}}
+                {{{ 0x10, 0x0C, 0xDD }, { 0xE0, 0x41, 0xC7 }, { 0x0F, 0xB2, 0x3A }, { 0xE0, 0x41, 0xC7 }, { 0x0F, 0xBF, 0x17 }}},
+                {{{ 0x0F, 0xB9, 0xFB }, { 0xE2, 0x0A, 0x1C }, { 0x0E, 0x95, 0xB4 }, { 0xE2, 0x0A, 0x1C }, { 0x0E, 0x4F, 0xB0 }}},
+                {{{ 0x0F, 0x46, 0xC0 }, { 0xE7, 0x71, 0x2C }, { 0x0D, 0x6A, 0xB6 }, { 0xE7, 0x71, 0x2C }, { 0x0C, 0xB1, 0x77 }}},
+                {{{ 0x13, 0x14, 0xD6 }, { 0xF4, 0x40, 0x0B }, { 0x04, 0xC9, 0x95 }, { 0xF4, 0x40, 0x0B }, { 0x07, 0xDE, 0x6C }}},
+                {{{ 0x0D, 0x4B, 0x9B }, { 0x06, 0x86, 0xF2 }, { 0x08, 0x37, 0xCD }, { 0x06, 0x86, 0xF2 }, { 0x05, 0x83, 0x68 }}}
             }                           // end of filter set
         },
         {                               // ----------     Equalization Set #11     ----------
