@@ -1,4 +1,4 @@
-/*
+ /*
  *  AudioHardwareConstants.h
  *  AppleOnboardAudio
  *
@@ -38,67 +38,72 @@
 
 #define kSoundEntryName	"sound"
 
-        //Sound Entry 
+// Sound Entry
+#define kSoundObjectsPropName		"sound-objects"
 
-#define kSoundObjectsPropName	"sound-objects"
-
-#define kInputObjEntryName	"input"
-#define kOutputObjEntryName	"output"
-#define kDetectObjEntryName	"detect"
-#define kFeatureObjEntryName	"feature"
-#define kInitObjEntryName	"init"	
-#define kMuxObjEntryName	"mux"	        
+#define kInputObjEntryName			"input"
+#define kOutputObjEntryName			"output"
+#define kDetectObjEntryName			"detect"
+#define kFeatureObjEntryName		"feature"
+#define kInitObjEntryName			"init"
+#define kMuxObjEntryName			"mux"
         
-#define kNumDetectsPropName	"#-detects"
-#define kNumFeaturesPropName	"#-features"
-#define kNumOutputsPropName	"#-outputs"
-#define kNumInputsPropName	"#-inputs"
-#define kModelPropName		"model"
+#define kNumDetectsPropName			"#-detects"
+#define kNumFeaturesPropName		"#-features"
+#define kNumOutputsPropName			"#-outputs"
+#define kNumInputsPropName			"#-inputs"
+#define kModelPropName				"model"
 
-#define kAnyInDetectObjName	"AnyInSenseBitsDetect"
-#define kInSenseBitsDetectObjName "InSenseBitsDetect"	
+#define kAnyInDetectObjName			"AnyInSenseBitsDetect"
+#define kInSenseBitsDetectObjName	"InSenseBitsDetect"	
+#define kGPIODetectObjName			"GPIODetect"
+#define kGPIOGenericDetectObjName	"GPIOGenericDetect"
+#define kGPIOPrioritizedDetectObjName	"GPIOPrioritizedDetect"
 
-#define kBitMaskPropName	"bit-mask"
-#define kBitMatchPropName	"bit-match"
-#define kDevicePropName		"device"
-#define kDeviceIDPropName	"device-id"
-#define kDeviceMaskPropName	"device-mask"
-#define kDeviceMatchPropName	"device-match"
-#define kDeviceTypePropName	"device_type"
-#define kIconIDPropName         "icon-id" 
-#define kPortChannelsPropName   "port-channels"
-#define kPortConnectionPropName "port-connection"
-#define kPortTypePropName       "port-type"      
-#define kNameIDPropName         "name-id" 
+#define kBitMaskPropName			"bit-mask"
+#define kBitMatchPropName			"bit-match"
+#define kDevicePropName				"device"
+#define kDeviceIDPropName			"device-id"
+#define kDeviceMaskPropName			"device-mask"
+#define kDeviceMatchPropName		"device-match"
+#define kDeviceTypePropName			"device_type"
+#define	kHasANDedResetPropName		"has-anded-reset"
+#define kIconIDPropName         	"icon-id"
+#define kPortChannelsPropName   	"port-channels"
+#define kPortConnectionPropName 	"port-connection"
+#define kPortTypePropName       	"port-type"
+#define kNameIDPropName         	"name-id"
 
-#define kOutputPortObjName	"OutputPort"
+#define kOutputPortObjName			"OutputPort"
 #define kOutputEQPortObjName		"OutputEQPort"
-#define kOutputDallasEQPortObjName	"OutputDallasEQPort"	
+#define kOutputDallasEQPortObjName	"OutputDallasEQPort"
+#define	kOutputMonoEQPortObjName	"OutputMonoEQPort"
 #define kGazIntSpeakerObjName		"Proj1Speaker"
 #define kGazSubwooferObjName		"Proj2Speaker"
 #define kWSIntSpeakerObjName		"Proj3Speaker"
 #define kGossSGSToneOutObjName		"Proj4Speaker"
 #define kKiheiSpeakerObjName		"Proj5Speaker"
 
-#define kAWACsModelName		"343S0140"
-#define kScreamerModelName	"343S0184"
-#define kBurgundyModelName	"343S0177"
-#define kDacaModelName		"353S0228"
-#define kTexasModelName		"355S0056"
+#define kAWACsModelName				"343S0140"
+#define kScreamerModelName			"343S0184"
+#define kBurgundyModelName			"343S0177"
+#define kDacaModelName				"353S0228"
+#define kTexasModelName				"355S0056"
+#define	kTexas2ModelName			"353S0303"
 
-#define kMuxPSurgeObjName 	"Proj1Mux"        
-#define kMuxAlchemyObjName   	"Proj2Mux"    
-#define kMuxHooperObjName    	"Proj3Mux"
-#define kMuxPExpressObjName	"Proj4Mux"    
-#define kMuxWSObjName         	"Proj5Mux"
-#define kMuxGossWingsAObjName  	"Proj6Mux" 
-#define kMuxGossWingsBObjName   "Proj7Mux"
-#define kMuxGossCanardObjName   "Proj8Mux"
-#define kMux101ObjName          "Proj9Mux"
-#define kMuxProgOutName         "MuxProgOut" 
+#define kMuxPSurgeObjName 			"Proj1Mux"        
+#define kMuxAlchemyObjName   		"Proj2Mux"    
+#define kMuxHooperObjName    		"Proj3Mux"
+#define kMuxPExpressObjName			"Proj4Mux"    
+#define kMuxWSObjName         		"Proj5Mux"
+#define kMuxGossWingsAObjName  		"Proj6Mux" 
+#define kMuxGossWingsBObjName   	"Proj7Mux"
+#define kMuxGossCanardObjName   	"Proj8Mux"
+#define kMux101ObjName          	"Proj9Mux"
+#define kMuxProgOutName         	"MuxProgOut" 
 
-#define kSourceMapPropName      "source-map"
-#define kSourceMapCountPropName "source-map-count"
+#define kSourceMapPropName      	"source-map"
+#define kSourceMapCountPropName 	"source-map-count"
 
 // Machine layout constants. All NewWorld machines have a device-id property
 // matching one of these layout constants for specifying the sound hardware layout.
@@ -121,19 +126,29 @@ enum {
 	layoutP29					=	16,
 	layoutWallStreet			=   17,
 	layoutP25					=	18,
-	layoutP53					=	19
+	layoutP53					=	19,
+	layoutP54					=	20,
+	layoutP57					=	21,
+	layoutP58					=	22,
+	layoutP62					=	23,
+	layoutP72					=	24,
+	layoutP92					=	25,
+	layoutP59					=	26,
+	layoutP57b					=	27,
+	layoutP73					=	28
 };
 
-        //Hardware type 
+// Hardware type 
 enum{
     kUnknownHardwareType,
     kGCAwacsHardwareType,
     kBurgundyHardwareType,
     kDACAHardwareType,
     kTexas3001HardwareType,
+    kTexas3004HardwareType
 };
 
-        //Kind of devices
+// Kind of devices
 enum {
     kSndHWInternalSpeaker	=	0x00000001,		// internal speaker present on CPU
     kSndHWCPUHeadphone		=	0x00000002,		// headphones inserted into CPU headphone port
@@ -146,27 +161,31 @@ enum {
     kSndHWModemRingDetect	=	0x00000100,		// modem ring detect
     kSndHWModemLineCurrent	=	0x00000200,		// modem line current
     kSndHWModemESquared		=	0x00000400,		// modem E squared
+	kSndHWLineInput			=	0x00000800,		//	line input device present
+	kSndHWLineOutput		=	0x00001000,		//	line output device present
 
-    kSndHWInputDevices		=       0x000000B0,		// mask to get input devices (excluding modems)
+    kSndHWInputDevices		=	0x000000B0,		// mask to get input devices (excluding modems)
     kSndHWAllDevices		=	0xFFFFFFFF		// all available devices
     
 };
 
-    // Codec kind
+// Codec kind
 enum {
     kSndHWTypeUnknown			=	0x00000000,		// unknown part
-    kSndHWTypeAWACs			=	0x00000001,		// AWACs part
+    kSndHWTypeAWACs				=	0x00000001,		// AWACs part
     kSndHWTypeScreamer			=	0x00000002,		// Screamer part
     kSndHWTypeBurgundy			=	0x00000003,		// Burgundy
-    kSndHWTypeUSB			=	0x00000004,		// USB codec on a wire...
-    kSndHWTypeDaca			= 	0x00000005,		// DAC3550A
+    kSndHWTypeUSB				=	0x00000004,		// USB codec on a wire...
+    kSndHWTypeDaca				= 	0x00000005,		// DAC3550A
     kSndHWTypeDigitalSnd		=	0x00000006,		// DigitalSnd virtual HW
     kSndHWTypeTumbler			=	0x00000007,		// Tumbler I2S with Equalizer & Dallas ID thing...
+	kSndHWTypeTexas2			=	0x00000008,		// Texas2 I2s with Equalizer & Dallas ID thing...
 
     kSndHWManfUnknown			=	0x00000000,		// unknown manufacturer (error during read)
     kSndHWManfCrystal			=	0x00000001,		// manufactured by crystal
     kSndHWManfNational			=	0x00000002,		// manufactured by national
-    kSndHWManfTI			=	0x00000003		// manufactured by texas instruments
+    kSndHWManfTI				=	0x00000003,		// manufactured by texas instruments
+    kSndHWManfMicronas			=	0x00000004		// manufactured by Micronas Intermetall
 };
 
 
@@ -190,7 +209,7 @@ enum {
 	kSndHWProgOutput4			=	0x00000010		// programmable output four
 };
 
-//Input Port constants
+// Input Port constants
 enum {
 	kSndHWInSenseNone			=	0x00000000,		// no input sense bits
 	kSndHWInSense0				=	0x00000001,		// input sense bit zero
@@ -216,6 +235,17 @@ enum {
   kSndHWInputNone               = 0     /* no input*/
 };
 
+enum GpioAddressSelector {
+	kHeadphoneMuteSel			=	'hmut',
+	kHeadphoneDetecteSel		=	'hcon',
+	kAmplifierMuteSel			=	'amut',
+	kSpeakerDetectSel			=	'dlas',
+	kCodecResetSel				=	'rset',
+	kLineInDetectSel			=	'ldet',
+	kLineOutMuteRefSel			=	'lmut',
+	kLineOutDetectSel			=	'lcon'
+};
+
 
 // Shift value to identify the control IDs
 #define DETECTSHIFT 1000
@@ -223,7 +253,7 @@ enum {
 #define INPUTSHIFT  1200
 
 
-    //PRAM read write values
+// PRAM read write values
 enum{
     kMaximumPRAMVolume 	= 7,
     kMinimumPRAMVolume	= 0,
@@ -237,4 +267,4 @@ enum{
 
 typedef UInt32 sndHWDeviceSpec;
 
-#endif
\ No newline at end of file
+#endif
